/* Copyright 1991 Digital Equipment Corporation.               */
/* All rights reserved. See copyright notice in attached file COPYRIGHT. */
/* Last modified on Sun Feb 24 16:10:51 1991 by burrows        */
#include <stdio.h>
#include <ctype.h>
#include <strings.h>
#include <sys/types.h>
#include <time.h>
#include "indexserver.h"

static char *months[] = {
	"jan", "feb", "mar", "apr", "may", "jun",
	"jul", "aug", "sep", "oct", "nov", "dec"
};

char *unparsedate (l, buf, lng)
long l;
char *buf;
int lng;
{
	if (lng)
		(void) sprintf (buf, "%2d %s %4d",
			(l&0x1f), months[(l >> 8) & 0xf], (l >> 12) & 0xfff);
	else
		(void) sprintf (buf, "%2d%s%02d",
			(l&0x1f), months[(l >> 8) & 0xf],
				((l >> 12) & 0xfff) % 100);
	return (buf);
}

int parsedate (s, pday, pmonth, pyear, str)
char *s;
int *pday;
int *pmonth;
int *pyear;
char *str;
{
	static int monthchars[256];
	static char lower[256];
	static int first = 1;
	register char *p, *q, *r;
	register int i, j;
	int day = -1;
	int month = -1;
	int year = -1;
	long t;
	struct tm *tm;
	int tmpmonth = -1;

	if (first) {
		for (i = 0; i < ELEMENTS (lower); i++)
			lower[i] = tolower (i);
		for (i = 0; i < ELEMENTS (months); i++) {
			monthchars[months[i][0]] |= 1<<i;
			monthchars[toupper (months[i][0])] |= 1<<i;
		}
		first = 0;
	}

	for (p = s; *p != '\0' && year < 0; ) {
		if (isdigit (*p & 0xff)) {
			if (month >= 0)
				year = atoi (p);
			else
				day = atoi (p);
		} else 
			month = -1;

		if ((j = monthchars[*p & 0xff]) != 0) {
			tmpmonth = -1;
			for (i = 0; i < ELEMENTS (months); i++) {
				if ((j & (1 << i)) != 0) {
					q = months[i];
					r = p;
					while (*q!='\0' && *q==lower[*r & 0xff])
						q++, r++;
					if (*q == '\0') {
						month = i;
						break;
					} else if (!isalpha (*r)) {
						if (tmpmonth == -1)
							tmpmonth = i;
						else
							tmpmonth = -2;
					}
				}
			}
			if (month == -1 && tmpmonth >= 0)
				month = tmpmonth;
		}

		if (isdigit (*p & 0xff)) {
			while (isdigit (*p & 0xff))
				p++;
		} else {
			while (*p != '\0' && !isspace (*p & 0xff) && 
					    !isdigit (*p & 0xff))
				p++;
		}
		while (*p != '\0' && isspace (*p & 0xff))
			p++;
	}

	if (year < 0 && month < 0 && day < 0 &&
			*s != '\0' && strcmp (s, "today") != 0)
		return (0);

	if (year < 0) {
		(void) time ((long *) &t);
		tm = localtime (&t);

		if (month < 0) {
			if (day < 0) {
				day = tm->tm_mday;
				month = tm->tm_mon;
				year = tm->tm_year + 1900;
			} else if (day > 31) {
				year = day;
				day = -1;
			} else {
				month = tm->tm_mon;
				year = tm->tm_year + 1900;
				if (day > tm->tm_mday) {
					if (month == 0) {
						month = 11;
						year--;
					} else
						month--;
				}
			}
		} else {
			year = tm->tm_year + 1900;
			if (month > tm->tm_mon)
				year--;
		}
	}

	if (day == 0 || day > 31 && month > 11 || year > 4095)
		return (0);

	if (year >= 0) {
		if (year < 60)
		    year += 2000;
		else if (year < 100)
			year += 1900;
	}
	*pday = day;
	*pmonth = month;
	*pyear = year;
	if (month < 0)
		str[0] = '\0';
	else
		(void) strcpy (str, months[month]);
	return (1);
}
